/*
 * File:   main.c
 * Author: Denshi Usagi
 *
 * Created on 2019/03/18, 21:36
 */

// PIC12F1822 Configuration Bit Settings

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = ON        // Internal/External Switchover (Internal/External Switchover mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = ON         // Low-Voltage Programming Enable (Low-voltage programming enabled)

#include <xc.h>
#include <pic12f1822.h>

typedef unsigned char		BYTE;
typedef unsigned short int	WORD;
typedef unsigned long int	DWORD;
typedef enum _BOOL { FALSE = 0, TRUE }	BOOL;

typedef union {
    struct {
		BYTE			:8;
		BYTE			:4;
		BYTE D12		:1;
		BYTE D13		:1;
		BYTE D14		:1;
		BYTE D15		:1;
	};
    struct {
		BYTE 			:1;
		BYTE MODE		:1;
		BYTE 			:1;
		BYTE DIV2		:1;
		BYTE 			:1;
		BYTE OPBITEN	:1;
		BYTE SLEEP12	:1;
		BYTE SLEEP1		:1;
		BYTE RESET		:1;
		BYTE 			:1;
		BYTE PSEL		:1;
		BYTE FSEL		:1;
		BYTE HLB		:1;
		BYTE B28		:1;
		BYTE			:1;
		BYTE			:1;
	};
	WORD wValue;
} AD9833reg_t;

#define AD9833_SYNC		LATA5
#define AD9833_SCLK		LATA4
#define AD9833_SDAT		LATA2

//=================================================================

static void AD9833_Reset(void);
static void AD9833_SetFreq(DWORD dwVal);
static void AD9833_SetPhase(WORD wVal);
static void AD9833_Start(BOOL bSquare);
static void AD9833_Send(WORD wData);

// FOSCが低いので待つ必要はなし
#define DelayUs(x)
#define DelayNs(x)

static AD9833reg_t AD9833;

//=================================================================

// ●メイン
void main(void)
{
	// ポート初期化
	LATA = 0;
	
	AD9833_SYNC = 1;
	AD9833_SCLK = 1;
	AD9833_SDAT = 0;

	TRISA5 = 0;

	TRISA4 = 0;
	ANSA4 = 0;

	TRISA2 = 0;
	ANSA2 = 0;
	
	AD9833_Reset();

	// 周波数設定
	//AD9833_SetFreq(0x51EB852); // 8MHz
	//AD9833_SetFreq(0x28F5C29); // 4MHz
	//AD9833_SetFreq(0x147AE14); // 2MHz
	AD9833_SetFreq(0x0A3D70A); // 1MHz
	//AD9833_SetFreq(0x04A8C15); // 455KHz
	
	// 出力開始
	AD9833_Start(FALSE);
	
	for(;;);

	return;
}

//  ●AD9833初期化
static void AD9833_Reset(void)
{
	AD9833.wValue = 0;

	AD9833.B28 = 1; // 常に28bit書き込み
	AD9833.RESET = 1;
	AD9833.SLEEP1 = 1; // Internal clock disabled
	AD9833.SLEEP12 = 1; // DAC powered down
	
	AD9833_Send(AD9833.wValue);
}

// ●AD9833周波数設定
static void AD9833_SetFreq(DWORD dwVal)
{
	AD9833reg_t Reg1, Reg2;

	Reg1.wValue = dwVal & 0x3FFF;
	Reg2.wValue = dwVal >> 14;
	
	// FREQ 0/1 交互に利用する
	if (AD9833.FSEL == 0)
	{
		// FREQ1へ書き込む
		AD9833.FSEL = 1;
		Reg1.D15 = 1;
		Reg2.D15 = 1;
	}
	else
	{
		// FREQ0へ書き込む
		AD9833.FSEL = 0;
		Reg1.D14 = 1;
		Reg2.D14 = 1;
	}

	AD9833_Send(Reg1.wValue);
	AD9833_Send(Reg2.wValue);

	// FREQ0/1切り替え
	AD9833_Send(AD9833.wValue);
}

// ●AD9833位相設定
static void AD9833_SetPhase(WORD wVal)
{
	AD9833reg_t Reg;
	Reg.wValue = wVal;

	// 常にPHASE0へ書き込む
	Reg.D15 = 1;
	Reg.D14 = 1;
	Reg.D13 = 0;

	AD9833_Send(Reg.wValue);
}

// ●AD9833動作開始
static void AD9833_Start(BOOL bSquare)
{
	AD9833.RESET = 0;
	AD9833.SLEEP1 = 0;
	AD9833.SLEEP12 = 0;
	
	if (bSquare)
	{
		AD9833.SLEEP12 = 1;
		AD9833.OPBITEN = 1;
		AD9833.DIV2 = 1;
	}
	
	AD9833_Send(AD9833.wValue);
}

// ●AD9833への送信
static void AD9833_Send(WORD wData)
{
	AD9833_SYNC = 0;

	int nCount = 0;
	do
	{
		AD9833_SDAT = (wData & 0x8000)? 1 : 0;
		DelayNs(5); // t7, t9
		
		AD9833_SCLK = 0;
		DelayNs(10); // t6
		AD9833_SCLK = 1;
		DelayNs(5); // t5-t9
		
		wData <<= 1;
	}
	while (++nCount < 16);
	
	AD9833_SYNC = 1;
	DelayNs(5); // t11
}
